// Подключаем библиотеку для создания дополнительных последовательных (Serial) портов.
#include <SoftwareSerial.h>
//Создаем последовательный порт на пинах 13-чтение и 2-передача.
SoftwareSerial BTSerial(14, 15); // RX, TX
// Переменная для приема данных по Bluetooth.
char bt_input;

// Подключаем библиотеку управления сервомоторами.
#include <Servo.h>
// Подключаем библиотеку управляющую дальномером.
#include "Ultrasonic.h"
// Инициализируем дальномер
// Контакт Trig соединен с 5 портом, Echo с 6 портом.
Ultrasonic sonar(5, 6);
// Создаем сервомотор поворта головы.
Servo neck;
// Объявляем переменные для хранения состояния двух моторов.
int motor_L1, motor_L2, input_L;
int motor_R1, motor_R2, input_R;
// Константы - постоянные значения для уточнения углов.
const int left_ang = 168;
const int front_ang = 98;
const int right_ang = 28;
// Временные константы служат для точного задания времени на поворот, разворот, движение вперед
// в миллисекундах.
const int time_90 = 390;
const int time_180 = 750;
const int time_10cm = 220;
// Номера портов к которым подключены датчики препятствия.
const int  Front1 = 2, Front2 = 12, Right = 13;
/////////////
int schet_rejim; // переменная служит для переключения режимов работы.
int switch_rejim; // переменная служит для переключения режимов работы.
int time_alone;

//==================================================
// Функция инициализации уравления моторами.
void setup_motor_system(int L1, int  L2, int iL, int  R1, int R2, int iR)
{
  // Заносим в переменные номера контактов (пинов) Arduino.
  motor_L1 = L1; motor_L2 = L2; input_L = iL;
  // Для левых и правых моторов машинки.
  motor_R1 = R1; motor_R2 = R2; input_R = iR;
  // Переводим указанные порты в состояние вывода данных.
  pinMode(motor_L1, OUTPUT);
  pinMode(motor_L2, OUTPUT);
  pinMode(input_L, OUTPUT);
  pinMode(motor_R1, OUTPUT);
  pinMode(motor_R2, OUTPUT);
  pinMode(input_R, OUTPUT);
}

// Функция задает скорость двигателя.
void setspeed(int LeftSpeed, int RightSpeed)
{
  // Задаем ширину положительного фронта от 0 до 255.
  analogWrite(input_L, LeftSpeed);
  analogWrite(input_R, RightSpeed);
  // Чем больше, тем интенсивнее работает мотор.
}
// движение вперед.
void forward()
{
    // Если двигатель будет работать не в ту сторону,
    // поменять на нем контакты местами.
    digitalWrite(motor_L1, HIGH);
    digitalWrite(motor_L2,LOW);
    digitalWrite(motor_R1, HIGH);
    digitalWrite(motor_R2,LOW);
}
// Поворот налево с блокировкой левых колес.
void forward_left()
{
  // блокируем вращение левых колес.
    digitalWrite(motor_L1, LOW); 
    digitalWrite(motor_L2, LOW);
   // правые колеса вращаются.
    digitalWrite(motor_R1, HIGH);
    digitalWrite(motor_R2, LOW);
}
// Поворот налево.
void left()
{
  // левые колеса вращаются назад
    digitalWrite(motor_L1, LOW); 
    digitalWrite(motor_L2, HIGH);
   // правые колеса вращаются.
    digitalWrite(motor_R1, HIGH);
    digitalWrite(motor_R2, LOW);
}
// Поворот направо с блокировкой правых колес.
void forward_right()
{
  // левые колеса вращаются.
    digitalWrite(motor_L1, HIGH); 
    digitalWrite(motor_L2, LOW);
    // блокируем вращение правых колес.
    digitalWrite(motor_R1, LOW);
    digitalWrite(motor_R2, LOW);
}
void right()
{
  // левые колеса вращаются.
    digitalWrite(motor_L1, HIGH); 
    digitalWrite(motor_L2, LOW);
    //правые колеса вращаются назад.
    digitalWrite(motor_R1, LOW);
    digitalWrite(motor_R2, HIGH);
}
// Включаем движение назад.
void backward()
{
    // Меняем направление вращения двигателей.
    digitalWrite(motor_L2, HIGH);
    digitalWrite(motor_L1,LOW);
    digitalWrite(motor_R2, HIGH);
    digitalWrite(motor_R1,LOW);
}
void backward_left()
{
    // Меняем направление вращения двигателей.
    digitalWrite(motor_L2, LOW);
    digitalWrite(motor_L1,LOW);
    digitalWrite(motor_R2, HIGH);
    digitalWrite(motor_R1,LOW);
}

void backward_right()
{
    // Меняем направление вращения двигателей.
    digitalWrite(motor_L2, HIGH);
    digitalWrite(motor_L1,LOW);
    digitalWrite(motor_R2, LOW);
    digitalWrite(motor_R1,LOW);
}
// Остановка
void _stop()
{
    // Блокируем все колеса.
    digitalWrite(motor_L2, LOW);
    digitalWrite(motor_L1,LOW);
    digitalWrite(motor_R2, LOW);
    digitalWrite(motor_R1,LOW);
}

void avtoroute()
{
  boolean d_Front1, d_Front2, d_Right;
  d_Front1 = digitalRead(Front1); d_Front2 = digitalRead(Front2); d_Right = digitalRead(Right);

  // Если ни один датчик не сработал.
  if (d_Front1 && d_Front2 && d_Right)
  {
    forward_right();//подворот вправо.
    delay(time_90 / 9);
    forward(); // едем вперед.
    delay(time_10cm / 2);
  }
  else
  {
    //Если сработал один из передних датчиков.
    if ((!d_Front1) || (!d_Front2))
    {
      // поворачиваем направо на 90 градусов.
      right(); // поворачиваем направо на 90 градусов.
      delay(time_90);
    }
    else
    { // Если сработал правый датчик.
      forward_left();//подворот влево.
      delay(time_90 / 9);
      forward(); // едем вперед.
      delay(time_10cm / 2);

    }
  }
}
//==================================================

void setup()
{
  // Инициализируем сервомотор, управление 9 портом.
  //neck.attach(9);
  // Заносим в переменные номера контактов (пинов) Arduino.
  // Для левых и правых моторов машинки.
  setup_motor_system(3, 4, 11, 7, 8, 10);
  // Инициализируем порты датчиков препятствия.
  pinMode(Front1, INPUT);
  pinMode(Front2, INPUT);
  pinMode(Right, INPUT);
  // Двигатели запущены.
  setspeed(255, 255);
  // Устанавливаем скорость передачи данных для НС-05 (Bluetooth модуль).
  BTSerial.begin(9600);
  // Переключаем A0 в двоичный режим работы, на передачу.
  pinMode(16, OUTPUT);
  // Устанавливаем скорость передачи данных по кабелю.
  // Порт компьютера
  Serial.begin(9600);
  // переводим HC-05 в нормальный режим работы.
  digitalWrite(16, 0);
  time_alone=0;
  switch_rejim = 0;
}
// Основная программа.
void loop()
{
  //Остановка если связь потеряна
  if(time_alone>5000){_stop();} 
  if (switch_rejim == 1)
  {
    avtoroute();
  }
  else if (BTSerial.available())
  {
    time_alone=0;
    // Читаем команды и заносим их в переменную. char преобразует код символа команды в символ.
    bt_input = (char)BTSerial.read();
    // Отправляем команду в порт, чтобы можно было их проверить в "Мониторе порта".
    Serial.println(bt_input);
    switch (bt_input) {
      // Вперед
      case 'F':
        forward();
        break;
      // Назад
      case 'B':
        backward();
        break;
      // Влево
      case 'L':
        left();
        break;
      // Вправо
      case 'R':
        right();
        break;
      // Прямо и влево
      case 'G':
        forward_left();
        break;
      // Прямо и вправо
      case 'I':
        forward_right();
        break;
      // Назад и влево
      case 'H':
        backward_left();
        break;
      // Назад и вправо
      case 'J':
       backward_right();
        break;
      // Стоп
      case 'S':
        _stop();
        break;
      // Скорость 0%
      case '0':
        setspeed(0, 0);
        break;
      // Скорость 10%
      case '1':
        setspeed(25, 25);
        break;
      // Скорость 20%
      case '2':
        setspeed(50, 50);
        break;
      // Скорость 30%
      case '3':
        setspeed(75, 75);
        break;
      // Скорость 40%
      case '4':
        setspeed(100, 100);
        break;
      // Скорость 50%
      case '5':
        setspeed(125, 125);
        break;
      // Скорость 60%
      case '6':
        setspeed(150, 150);
        break;
      // Скорость 70%
      case '7':
        setspeed(175, 175);
        break;
      // Скорость 80%
      case '8':
        setspeed(200, 200);
        break;
      // Скорость 90%
      case '9':
        setspeed(225, 225);
        break;
      // Скорость 100%
      case 'q':
        setspeed(255, 255);
        break;
      case 'X':
      //  switch_rejim = 1;
        break;
      case 'x':
    //    switch_rejim = 0;
        break;
      case 'D':
        _stop(); switch_rejim = 0;
        break;
    }
  }
  else time_alone++;
}



